package main

import (
	"context"
	"io/ioutil"
	"log"
	"time"

	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"

	"database/sql"

	"fmt"

	"github.com/go-sql-driver/mysql"
	_ "github.com/go-sql-driver/mysql"
)

var client mongo.Client
var colUsers *mongo.Collection
var mongoCtx context.Context
var mariaDB *sql.DB

func init() {
	openMongoDBConnection()
	openMariaDBConnection()
}

func openMongoDBConnection() {
	// Mongo DB setup
	clientOptions := options.Client().ApplyURI("mongodb://" + config.MongoDbUser + ":" + config.MongoDbPass + "@" + config.MongoDbHost + ":" + config.MongoDbPort)
	client, err := mongo.NewClient(clientOptions)
	if err != nil {
		log.Fatal("DB Error: ", err)
	}
	// Cancel if Timeout
	ctx, cancel := context.WithTimeout(context.Background(), 20*time.Second)
	defer cancel()

	// Connect to DB
	err = client.Connect(ctx)
	if err != nil {
		log.Fatal("DB Error: ", err)
	}

	colUsers = client.Database("bs1in").Collection("users")
}

func openMariaDBConnection() {
	dbConf := mysql.Config{
		User:                 config.MariaDbUser,
		Passwd:               config.MariaDbPass,
		Net:                  "tcp",
		Addr:                 config.MariaDbHost + ":" + config.MariaDbPort,
		AllowNativePasswords: true, // Without this it did not work...
		MultiStatements:      true, // To read sql files
	}
	var err error
	mariaDB, err = sql.Open("mysql", dbConf.FormatDSN())
	if err != nil {
		log.Fatal("MariaDB Connection Error: ", err)
	}

	err = mariaDB.Ping()
	if err != nil {
		log.Fatal("MariaDB not reachable (Ping Error): ", err)
	}

	var version string
	mariaDB.QueryRow("SELECT VERSION()").Scan(&version)
	log.Println("Connected to:", version)
}

func createMariaDbUser(uid string, pw string) bool {
	createUserStmt := `CREATE OR REPLACE USER '%s'@'%%' IDENTIFIED BY '%s';`
	createUserStmt = fmt.Sprintf(createUserStmt, uid, pw)
	log.Println(createUserStmt)
	_, err := mariaDB.Exec(createUserStmt)

	if err != nil {
		log.Println("Could not create database user: ", err)
		return false
	}

	return true
}

func createMariaDbDatabasesForUser(uid string) bool {
	// _, err := mariaDB.Exec(`DROP USER IF EXISTS '?';
	// CREATE USER '?'@'%';
	// DROP DATABASE IF EXISTS ?_W19_20;
	// CREATE DATABASE ?_W19_20;
	// GRANT ALL ON ?_W19_20.* TO '?'@'%' IDENTIFIED BY '?';`, uid, uid, uid, uid, uid, uid, pw)
	sqlFilePath := "../../sql/"
	files, err := ioutil.ReadDir(sqlFilePath)
	if err != nil {
		log.Println("Could not read sql files: ", err)
		return false
	}
	for _, file := range files {
		log.Println("Processing ", file.Name())

		sqlFromFile, err := ioutil.ReadFile(sqlFilePath + file.Name())
		if err != nil {
			log.Println("Could not read sql file ", file.Name(), ": ", err)
			return false
		}

		createDatabaseStmt := fmt.Sprintf(string(sqlFromFile), uid)
		// log.Println(createDatabaseStmt)
		_, err = mariaDB.Exec(createDatabaseStmt)
		if err != nil {
			log.Println("Could not process sql file ", file.Name(), ": ", err)
			return false
		}

	}
	return true
}
